@@ -1,4 +1,5 @@
-#coding:utf-8
+# coding:utf-8
+
 import base64
 from copy import copy
 import json
@@ -8,9 +9,12 @@ from decimal import Decimal
 
 from django.utils.encoding import force_text
 
-from .enums import (WarehouseType, AssortmentType, TransportOrderRoleContext, WaybillRoleContext)
-from .models import (FilterItem, SortItem, Address, ElvisModel, TransportOrderListPage, TransportOrder, TransportOrderStatusInfo,
-                     TimberWarehouse, Waybill, WaybillStatusInfo, WaybillListPage, TimberAssortment, FineMeasurementFile)
+from .enums import WarehouseType, AssortmentType, TransportOrderRoleContext, WaybillRoleContext
+from .models import (
+    FilterItem, SortItem, Address, ElvisModel, TransportOrderListPage, TransportOrder, TransportOrderStatusInfo,
+    TimberWarehouse, Waybill, WaybillStatusInfo, WaybillListPage, TimberAssortment, FineMeasurementFile,
+)
+from .utils import DATE_PREFIX, DATE_SUFFIX, ELVIS_TIMEZONE
 
 
 class ElvisException(Exception):
@@ -18,8 +22,41 @@ class ElvisException(Exception):
         self.message = message
         self.raw = raw
 
+    @classmethod
+    def reformat_elvis_exception_text(cls, value, indent=1):
+        new_value = value
+
+        if isinstance(value, dict):
+            new_value = "\n"
+            for key in value:
+                new_value = "%s%s`%s`:%s\n" % (
+                    new_value,
+                    "    " * indent,
+                    key,
+                    cls.reformat_elvis_exception_text(
+                        value[key], indent=indent+1
+                    ),
+                )
+
+        elif isinstance(value, list):
+            new_value = "\n"
+            for index, item in enumerate(value):
+                new_value = "%s%s[%s]:%s\n" % (
+                    new_value,
+                    "    " * indent,
+                    index,
+                    cls.reformat_elvis_exception_text(
+                        item, indent=indent+1
+                    ),
+                )
+
+        elif hasattr(value, "replace"):
+            new_value = value.replace(u"\\u000d\\u000a", "\n")
+
+        return new_value
+
     def __str__(self):
-        return "ElvisException %s: %s" % (self.message, self.raw)
+        return "ElvisException %s: %s" % (self.message, self.reformat_elvis_exception_text(self.raw))
 
 
 class ElvisEncoder(json.JSONEncoder):
@@ -36,7 +73,18 @@ class ElvisEncoder(json.JSONEncoder):
             return float(obj)
 
         if isinstance(obj, datetime):
-            return '/Date(%d)/' % ((obj - datetime(1970, 1, 1)).total_seconds() * 1000)
+            # Elvis timestamp offsets are from Tallinn time
+            timezone_offset = obj.utcoffset() - obj.astimezone(ELVIS_TIMEZONE).utcoffset()
+
+            timezone_offset_hours = timezone_offset.total_seconds() // 3600
+            timezone_offset_minutes = (timezone_offset.total_seconds() - timezone_offset_hours * 3600) // 60
+            return '%s%d+%02d%02d%s' % (
+                DATE_PREFIX,
+                (obj - datetime(1970, 1, 1)).total_seconds() * 1000,
+                timezone_offset_hours,
+                timezone_offset_minutes,
+                DATE_SUFFIX,
+            )
 
         if not isinstance(obj, ElvisEncoder.ELVIS_OBJECTS):
             return super(ElvisEncoder, self).default(obj)
@@ -56,13 +104,19 @@ class ElvisEncoder(json.JSONEncoder):
 
 
 class ElvisClient(object):
-    def __init__(self, api_url, person_code, certificate_pass="", session_token=None):
+    DEFAULT_REQUEST_TIMEOUT = 120  # 120 seconds
+
+    def __init__(
+        self, api_url, person_code, certificate_pass="",
+        session_token=None, request_timeout=DEFAULT_REQUEST_TIMEOUT,
+    ):
         self.api_url = api_url
 
         self.person_code = person_code
         self.certificate = None
         self.certificate_password = certificate_pass or ""
         self.session_token = session_token or None
+        self.request_timeout = request_timeout
 
         assert self.person_code, "Person code must be provided"
 
@@ -105,31 +159,33 @@ class ElvisClient(object):
             headers['Authorization'] = self.session_token
 
         if method.lower() == "post":
-            result = func(self.api_url % endpoint, data=json.dumps(attrs, cls=ElvisEncoder), headers=headers)
+            result = func(self.api_url % endpoint, data=json.dumps(attrs, cls=ElvisEncoder), headers=headers,
+                          timeout=self.request_timeout)
         else:
-            result = func(self.api_url % endpoint, params=attrs, headers=headers)
+            result = func(self.api_url % endpoint, params=attrs, headers=headers, timeout=self.request_timeout)
+
+        error_message = ""
+        try:
+            json_data = json.loads(result.text)
+        except ValueError:
+            error_message = "not json"
+            json_data = result.text
 
         if result.status_code == 200:
-            text = result.text
-
-            try:
-                return {
-                    "Success": True,
-                    "raw": json.loads(text),
-                }
-            except ValueError:
-                return {
-                    "Success": False,
-                    "message": "not json",
-                    "raw": text,
-                }
-
+            success = True
         else:
-            return {
-                "Success": False,
-                "message": "Bad status code: %d" % result.status_code,
-                "raw": result.text,
-            }
+            error_message = "Bad status code: %d" % result.status_code,
+            success = False
+
+        result = {
+            "Success": success,
+            "raw": json_data,
+        }
+
+        if error_message:
+            result["message"] = error_message
+
+        return result
 
     # ENDPOINTS
 
